@@ -33,6 +33,9 @@
 #ifdef HAVE_SYS_MOUNT_H
 #include <sys/mount.h>
 #endif
+#ifdef HAVE_SYS_POLL_H
+#include <sys/poll.h>
+#endif
 #endif
 #include <stdio.h>
 #include <unistd.h>
@@ -73,6 +76,8 @@
 #endif
 #endif
 
+#define STAT_TIMEOUT_SECONDS 3
+
 
 /* Ideally this should not nonblocking stat, since that can block on
  * downed NFS mounts forever, however there seems to be no good way
@@ -86,8 +91,12 @@ _gnome_vfs_unix_mount_get_unix_device (GnomeVFSUnixMount *mount)
 	dev_t unix_device;
 	pid_t pid;
 	int pipes[2];
-	fd_set read_fds;
+#ifdef HAVE_POLL
+	struct pollfd poll_fd;
+#else
 	struct timeval tv;
+	fd_set read_fds;
+#endif
 	int res;
 	int status;
 
@@ -98,6 +107,8 @@ _gnome_vfs_unix_mount_get_unix_device (GnomeVFSUnixMount *mount)
 	pid = fork ();
 
 	if (pid == -1) {
+		close (pipes[0]);
+		close (pipes[1]);
 		return 0;
 	}
 
@@ -119,12 +130,14 @@ _gnome_vfs_unix_mount_get_unix_device (GnomeVFSUnixMount *mount)
 				write (pipes[1], (char *)&statbuf.st_dev, sizeof (dev_t));
 			}
 		} else {
-			close (pipes[0]);
 			close (pipes[1]);
 		}
 
 		_exit (0);
 	} else {
+		/* Parent */
+		close (pipes[1]);
+		
         wait_again:
 		if (waitpid (pid, &status, 0) < 0) {
 			if (errno == EINTR)
@@ -134,20 +147,25 @@ _gnome_vfs_unix_mount_get_unix_device (GnomeVFSUnixMount *mount)
 			else
 				g_warning ("waitpid() should not fail in gnome_vfs_unix_mount_get_unix_device");
 		}
+
+
+		do {
+#ifdef HAVE_POLL
+			poll_fd.fd = pipes[0];
+			poll_fd.events = POLLIN;
+			res = poll (&poll_fd, 1, STAT_TIMEOUT_SECONDS*1000);
+#else
+			tv.tv_sec = STAT_TIMEOUT_SECONDS;
+			tv.tv_usec = 0;
+			
+			FD_ZERO(&read_fds);
+			FD_SET(pipes[0], &read_fds);
+			
+			res = select (pipes[0] + 1,
+				      &read_fds, NULL, NULL, &tv);
+#endif
+		} while (res == -1 && errno == EINTR);
 		
-		/* Parent */
-		close (pipes[1]);
-
-		FD_ZERO(&read_fds);
-		FD_SET(pipes[0], &read_fds);
-
-		/* Wait max one second */
-		tv.tv_sec = 2;
-		tv.tv_usec = 0;
-
-		res = select(pipes[0] + 1,
-			     &read_fds, NULL, NULL, &tv);
-
 		if (res > 0) {
 			if (read (pipes[0], (char *)&unix_device, sizeof (dev_t)) != sizeof (dev_t)) {
 				unix_device = 0; 
@@ -203,6 +221,7 @@ _gnome_vfs_get_current_unix_mounts (GList **return_list)
 	char *stat_file;
 	struct stat sb;
 	GnomeVFSUnixMount *mount_entry;
+	GHashTable *mounts_hash;
 	
 	read_file = get_mtab_read_file ();
 	stat_file = get_mtab_monitor_file ();
@@ -227,13 +246,34 @@ _gnome_vfs_get_current_unix_mounts (GList **return_list)
 		return TRUE;
 	}
 
+	mounts_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	
 	while ((mntent = getmntent (file)) != NULL) {
+		/* ignore any mnt_fsname that is repeated and begins with a '/'
+		 *
+		 * We do this to avoid being fooled by --bind mounts, since
+		 * these have the same device as the location they bind to.
+		 * Its not an ideal solution to the problem, but its likely that
+		 * the most important mountpoint is first and the --bind ones after
+		 * that aren't as important. So it should work.
+		 *
+		 * The '/' is to handle procfs, tmpfs and other no device mounts.
+		 */
+		if (mntent->mnt_fsname != NULL &&
+		    mntent->mnt_fsname[0] == '/' &&
+		    g_hash_table_lookup (mounts_hash, mntent->mnt_fsname)) {
+			continue;
+		}
+
 		mount_entry = g_new0 (GnomeVFSUnixMount, 1);
 
 		mount_entry->mount_path = g_strdup (mntent->mnt_dir);
 		mount_entry->device_path = g_strdup (mntent->mnt_fsname);
 		mount_entry->filesystem_type = g_strdup (mntent->mnt_type);
 		
+		g_hash_table_insert (mounts_hash,
+				     mount_entry->device_path,
+				     mount_entry->device_path);
 
 #if defined (HAVE_HASMNTOPT)
 		if (hasmntopt (mntent, MNTOPT_RO) != NULL) {
@@ -242,6 +282,7 @@ _gnome_vfs_get_current_unix_mounts (GList **return_list)
 #endif
 		*return_list = g_list_prepend (*return_list, mount_entry);
 	}
+	g_hash_table_destroy (mounts_hash);
 	
 	endmntent (file);
 	
